% Questo script simula la procedura di accesso base del protocollo MAC 802.11 all'interno 
% di un area indoor in cui si muovono casualmente n stazioni mobili.
% Come parametri in ingresso vengono forniti:
% 1) il numero delle stazioni mobili
% 2) il raggio di copertura massimo
% 3) la lunghezza del pacchetto
% 4) la durata della simulazione
%
% Come risultati in uscita si avranno:
% 1) il numero delle trasmissioni riuscite con successo
% 2) il numero delle collisioni dei pacchetti dei dati
% 3) il numero di trasmissioni non riuscite perch la stazione ricevente  fuori dal raggio
%    di copertura massimo della stazione inviante
% 4) il numero delle collisioni degli ACK
% 5) il numero degli ACK che non arrivano perch la stazione che deve riceverlo  fuori dal
%    raggio di copertura della stazione che glielo invia

'Simulation parameters _____ (default value)'

n=input('Number of Stations?_____(5) ');

if length(n)==0
   n=5;
end

r=input('Range of Radio?_____(3) ');

if length(r)==0
	r=3;
end

frame_size=input('Average Transmission Time (packet size)?______(16) ');
 
if length(frame_size)==0
	frame_size=16;
end

max_simutime=input('Simulation Time? (SlotTime=2)_____(600) ');% in ms

% Ci sar un loop basato su questo valore del tempo di simulazione (linea 153 con counter =1:max_simutime) 
% Ogni volta che viene chiamato, modifica_oggetti.m  eseguito per un loop che va da 1:n (dove 'n'  il numero dei nodi)

if length(max_simutime)==0
	max_simutime=600;
end

sss=input('Show graphic?______(y) ');

if length(sss)==0
	sss='y';
end

ph=0:0.02:2*pi;

if sss=='y'
	figure
	hold on
   j=sqrt(-1);
   
  for i=1:n
     h1(i)=plot(exp(j*ph)*(1+r),'EraseMode','xor','color','r','Marker','o','Markersize',5);
     % disegna una circonferenza con bordo rosso che rappresenta il nodo mobile e che pu essere
     % cancellata senza danneggiare le figure sottostanti (comando ErasMode)
     h2(i)=plot(exp(j*ph)*(1+r),'EraseMode','xor');
     % disegna il cerchietto esterno che rappresenta il nodo mobile
   
     h4(i)=plot(exp(j*ph)*(1+r),'EraseMode','xor');
	  % disegna la circonferenza che rappresenta il raggio di copertura massimo   
     h5(i)=plot(exp(j*ph)*(1+r),'EraseMode','xor','LineWidth',3,'color','r');
     % disegna la linea rossa che pu partire dal cerchietto rappresentante il nodo mobile
     % e che rappresenta l'invio del pacchetto dati
     h6(i)=plot(exp(j*ph)*(1+r),'EraseMode','xor','LineWidth',3,'color','g');
     % disegna la linea verde che pu partire dal cerchietto rappresentante il nodo mobile
     % e che rappresenta l'invio del pacchetto ACK
   
  end
  
  % vengono graficate sulla figura delle legende che spiegano cosa sono le linee visualizzate
  % durante la simulazione
	tempx=-(1+r)*1.2;
	tempy=(1+r)*1.2;
	offsetx=(1+r)/5;
	offsety=(1+r)/7;
	plot([tempx,tempx+offsetx],[tempy,tempy],'color','r','LineWidth',3);
	text(tempx+offsetx*1.2,tempy,'Pacchetti dati all''interno del raggio di copertura');
	plot([tempx,tempx+offsetx],[tempy-offsety,tempy-offsety],'color','m','LineWidth',3);
	text(tempx+offsetx*1.2,tempy-offsety,'Pacchetti dati fuori dal raggio di copertura');
	plot([tempx,tempx+offsetx],[tempy-offsety*2,tempy-offsety*2],'color','g','LineWidth',3);
   text(tempx+offsetx*1.2,tempy-offsety*2,'Pacchetti di ACK');
end

traffic=1.0;	% scalare che indica un certo valore di traffico
ACK_length=4;	% lunghezza dell'ACK in bytes
Sifs=2;			% durata del SIFS
SlotTime=2;		% durata di uno slot

% a,b,c,d sono dei parametri che vengono inseriti per facilitare le successive
% espressioni usate per gli spostamenti random della posizione delle stazioni mobili
a=0.001;
b=0.7;
c=0.0005;
d=0.997;

Difs=Sifs+2*SlotTime;	% formula del DIFS indicata dallo standard
state=zeros(1,n);			% inizializzazione a zero degli stati del sistema per ogni stazione
Timer=zeros(1,n);			% inizializzazione a zero del timer delle n stazioni mobili

pos_x=randn(1,n);		% vengono generate due matrici (1xn) che contengono numeri casuali con
pos_y=randn(1,n);		% distribuzione normale che rappresentano la posizione del mobile

pos_x_change=sqrt(a)*randn(1,n);  % piccolo spostamento delle posizioni delle stazioni
pos_y_change=sqrt(a)*rand(1,n);	% rispetto alle posizioni casuale assegnate

PCWmin=3;		% parametro minimo per la Contention Window
PCWmax=8;		% parametro massimo per la Contention Window
PCW=PCWmin*ones(1,n);	% inizializzazione della durata della Contention Window per ogni stazione
RC=zeros(1,n);		% inizializzazione a zero del retry counter
BC=zeros(1,n);		% inizializzazione a zero del back-off counter
range=r*ones(1,n);	% inizializzazione del raggio di copertura delle n stazioni
within=ones(n,n);		% inizializzazione della matrice (nxn) che indica quali stazioni sono
							% nel raggio di copertura della n-esima altra stazione
                     
% ciclo in cui si verifica se qualcuna delle n stazioni  fuori dal raggio di copertura di 
% qualche altra e in tal caso si assegna il valore zero all'elemento della matrice within
% che caratterizza le due matrici che "non si vedono"

for i=1:n
   
   for k=1:n
      
		if ((pos_x(i)-pos_x(k))^2+(pos_y(i)-pos_y(k))^2>range(i)^2)
         within(i,k)=0;
      end
      
   end
   
end

current_frame_length=zeros(1,n); % inizializzazione a zero del tempo di invio del frame corrente 
current_frame_dest=zeros(1,n);	% inizializzazione a zero delle stazioni destinatarie del frame
initial_cfl=zeros(1,n);		% inizializzazione a zero del current_frame_lenght iniziale
ACK_dest=zeros(1,n);			% inizializzazione a zero delle stazioni destinatarie dell'ACK
collision=0;		% inizializzazione a zero del numero delle collisioni
transmission=0;	% inizializzazione a zero del numero delle trasmissioni riuscite
unreachable=0;		% inizializzazione a zero del numero delle trasmissioni non riuscite perch
						% la stazione ricevente era fuori dal raggio di copertura della trasmittente
ACK_collision=0;		% inizializzazione a zero delle collisioni degli ACK
ACK_unreachable=0;	% inizializzazione a zero degli ACK non arrivati perch la stazione ricevente
							% era fuori dal raggio di copertura della stazione inviante

% INIZIO DEL LOOP PRINCIPALE

for counter=1:max_simutime
   
   t0=clock;	% inizializzazione di t0 all'ora corrente
   
   % istruzioni che provocano dei piccoli spostamenti casuali delle stazioni in entrambe le coordinate
	pos_x_change=b*pos_x_change+randn(1,n)*sqrt(c);
   pos_y_change=b*pos_y_change+randn(1,n)*sqrt(c);
   pos_x=pos_x*d+pos_x_change;
   pos_y=pos_y*d+pos_y_change;
   
   within=ones(n,n);		% nuova inizializzazione della matrice within (vedi linea 112)
   
   % ciclo identico a quello gi visto nelle linee 119-129
   for i=1:n
      
      for k=1:n
         
			if ((pos_x(i)-pos_x(k))^2+(pos_y(i)-pos_y(k))^2>range(i)^2)
         	within(i,k)=0;
         end
         
      end
      
   end
   
   sending=zeros(1,n);	% inizializzazione a zero della matrice delle stazioni che devono inviare
   
   % IL SISTEMA E' STATO DIVISO NEI SEGUENTI STATI CHE PERMETTONO DI ANALIZZARE LE VARIE
   % FASI DELLA SIMULAZIONE:
   %
   % -3: collisione di DATI?
	% -2: collisione di un ACK?
	% -1: invio di un ACK
	% 0: mezzo libero (niente da inviare)
	% 1: in attesa della media (cio ho qualcosa da inviare)
	% 2: media rilevata, in attesa che trascorri il DIFS
	% 3: Random back-off
	% 4: in attesa che trascorri il SIFS per poter inviare l'ACK
	% 5: invio avvenuto con successo
	% 6: invio ad una stazione occupata
	% 7: invio ad un nodo fuori dal raggio di copertura

	sending(state>=5 | state<=-1)=1;		% si  inviato un frame se state>=5 oppure state<=-1

   % B_media  una matrice (1xn) delle stazioni che devono inviare un pacchetto a una stazione
   % che sia all'interno del suo raggio di copertura, in pratica fa "carrier sense" cio serve  
   % a stabilire se il mezzo  libero oppure no: se B_media<1 allora viene rilevato libero il
   % mezzo attorno al raggio di copertura della stazione
   
   B_media=sending*within;  
   
   % in attesa della media - passaggio allo stato 2
   
   transit2=zeros(1,n);						% transit2 indica che siamo nello stato 1 (ho qualcosa da inviare)  
   transit2(state==1 & B_media<1)=1;	% e viene rilevato libero il mezzo attorno al raggio di copertura  
   
   %  state(transit2>0)=2;
   
   Timer(transit2>0)=Difs;	% la stazione rileva il mezzo libero ma deve attendere un DIFS
   
	transit1=zeros(1,n);						% transit1 indica che sono passato nello stato 2 perch ho rilevato
   transit1(state==2 & B_media>0)=1;	% che la stazione a cui devo inviare il frame  nel mio raggio di copertura
   
   Timer(state==2)=Timer(state==2)-1;	% il Timer del DIFS viene decrementato di 1
   
   % trascorso il DIFS - passaggio allo stato 3
   
   transit3=zeros(1,n);					% transit3 indica che sto nello stato 2 ed  trascorso il 
	transit3(state==2 & Timer<0)=1;	% DIFS, cio pu iniziare la fase di contesa con l'algortimo di Backoff
   state(transit1>0)=1;
   
   transit1(B_media>0 & state==3)=1;  % transit1 indica che inizia l'algoritmo di Backoff
   
   % state(transit1>0)=1;
   
   BC(state==3 & B_media<1)=BC(state==3 & B_media<1)-1;  % il timer di backoff viene decrementato di 1 se il  
   																		% mezzo  libero attorno al raggio di copertura
   
   % trascorso l'intervallo di Backoff - passaggio allo stato 5 
                                                         
   transit5=zeros(1,n);
   transit5(state==3 & BC<0)=1;	% transit5 indica che il timer di backoff  arrivato a zero
   										% e posso quindi inviare il frame di dati
                                 
   for i=1:n
      
	   if (state(i)==5 & B_media(current_frame_dest(i))>1)   % se una stazione invia un frame ad un'altra
	      state(i)=6;														% che deve inviare anch'essa, allora avviene
      end																	% il passaggio allo stato 6
      
		if (state(i)==5 & within(i,current_frame_dest(i))<1)	% se una stazione invia un frame ad un'altra
	      state(i)=7;														% che viene rilevata fuori dal suo raggio di copertura
      end																	% allora avviene il passaggio allo stato 7
      
   end
   
   current_frame_length(state>=5)=current_frame_length(state>=5)-1;	% se comunque avviene l'invio di un frame, viene
   																						% decrementata di 1 il suo tempo di invio
	transit0=zeros(1,n);
	transit0(state>=5 & current_frame_length<=0)=1;		% transit0 indica che il frame  stato inviato completamente
   PCW(transit0>0 & state>5)=PCW(transit0>0 & state>5)+1;	% in caso di invio del frame senza successo viene  
																				% incrementata di 1 la Contention Window
   PCW(PCW>PCWmax)=PCWmax;		% fissa comunque a PCWmax il valore massimo della Contention Window
   PCW(transit0>0 & state==5)=PCWmin;	% in caso di invio con successo del frame la Contention  
   												% Window torna al valore minimo
   
   %%%%%%%%%%%%%%%%%%%% Valori in uscita %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % vengono aggiornate le trasmissioni riuscite, le collisioni e le trasmissioni non riuscite   
   % perch la stazione ricevente era fuori dal raggio di copertura di quella inviante
   
	transmission=transmission+length(state(transit0>0 & state==5));
	collision=collision+length(state(transit0>0 & state==6));
   unreachable=unreachable+length(state(transit0>0 & state==7));
   
   % quando giunge un pacchetto alla destinazione, questa deve attendere un SIFS priva di 
   % inviare l'ACK - passaggio allo stato 4
   
   transit4=zeros(1,n);
   
   % ciclo che inizializza la destinazione dell'ACK nel caso in cui sia inviato con 
   % un frame destinato ad una certa stazione
   for i=1:n
      
		if (state(i)==5 & transit0(i)>0)
         ii=current_frame_dest(i);
         Timer(ii)=Sifs;
         transit4(ii)=1;
         ACK_dest(ii)=i;
      end
      
   end
   
   % 
   
	Timer(state==4 & B_media>0)=Sifs;
   Timer(state==4 & B_media<1)=Timer(state==4 & B_media<1)-1; % il timer dell'ACK viene decrementato di 1 se il
   																			  % mezzo  libero attorno al raggio di copertura
   % invio del pacchetto di ACK
                                                              
	transit_1=zeros(1,n);
	transit_1(state==4 & Timer<0)=1;		% transit_1 indica che il SIFS  scaduto e posso quindi inviare l'ACK
   Timer(transit_1>0)=ACK_length;
   
   % ciclo in cui vengono considerati le possibilit di collisione di ACK (stato -2) 
   % o di dati (stato -3) in caso di mancata ricezione dell'ACK da parte della stazione inviante il frame
   
   for i=1:n
      
	    if (state(i)==-1)
          ii=ACK_dest(i);
          
         if (B_media(ii)>1)
             state(i)=-2;
         end
          
        	if (within(i,ii)<1)
	             state(i)=-3;
         end
             
       end
         
   end   
     
	Timer(state<=-1)=Timer(state<=-1)-1;	% viene decrementato di 1 il timer di invio dell'ACK
   transit0(state<=-1 & Timer<0)=1;		% transit0 indica che l'ACK  stato inviato completamente
   
   %%%%%%%%%%%%%%%%%%%% Valori in uscita %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
   % aggiornamento delle collisioni dell'ACK e delle trasmissioni non concluse perch 
   % l'ACK non  arrivato poich fuori dal raggio di copertura
   
   ACK_collision=ACK_collision+length(state(state==-2 & transit0>0));
   ACK_unreachable=ACK_unreachable+length(state(state==-3 & transit0>0));
   
   % Quando, ricevuto l'ACK, si conclude la trasmissione, il mezzo torna libero (stato 0) 
   
	state(transit0>0)=0;
   temp=rand(1,n); 	% temp  una matrice random che rappresenta il tempo totale della trasmissione
   
   % passaggio allo stato 1
   
   transit1(state==0 & temp<traffic)=1;	% avviene il passaggio allo stato 1 dallo stato 0 (mezzo libero)
   													% se viene rilevato un livello di traffico che permetta la durata 
   													% di una trasmissione completa (dato caricato nel registro NAV)
   temp=Sifs+rand(1,n)*frame_size*2;  % temp  il tempo totale della trasmissione (DATA+SIFS+ACK) che varia
   											  % in base alle condizioni del traffico della rete
	current_frame_length(transit1>0 & state==0)=temp(transit1>0 & state==0); % tempo di invio del frame = temp
   initial_cfl(transit1>0 & state==0)=current_frame_length(transit1>0 & state==0); % inizializzazione del tempo di
   																										  % invio del frame
   % viene cambiato il valore delle stazioni destinatarie del frame                                                                                 
                                                                                   
   temp=ceil(rand(1,n)*n);  % ceil arrotonda per eccesso all'intero pi vicino
   current_frame_dest(transit1>0 & state==0)=temp(transit1>0 & state==0);
   
   % ciclo usato per variare le stazioni destinatarie del frame per ognuna delle stazioni invianti
   
   for i=1:n
      
		while (current_frame_dest(i)==i | current_frame_dest(i)==0)
		       current_frame_dest(i)=ceil(rand(1,1)*n);
      end
          
   end
   
   temp=floor(rand(1,n).*(2.^PCW));  % floor arrotonda per difetto all'intero pi vicino
   											 % .^ eleva a potenza tutti gli elementi della matrice PCW
   BC(transit1>0 & state==0)=temp(transit1>0 & state==0)*SlotTime;  % l'intervallo di Backoff viene suddiviso in 
   % valori scelti a caso multipli di uno SLOT
   
   % gli stati sono cambiati qui
   
	state(transit1>0)=1;
	state(transit2>0)=2;
   state(transit3>0)=3;
  	state(transit5>0)=5;
	state(transit4>0)=4;
	state(transit_1>0)=-1;
   state(transit0>0)=0;
   
	if sss=='y'
		modifica_oggetti	  % se il dato di ingresso  'y' viene eseguito il file modifica_oggetti.m
      drawnow		        % che descrive il cambiamento degli oggetti grafici che rappresentano le stazioni mobili
      
		while etime(clock,t0)<0.1
		      i=1;
      end
         
   end
   
   %  input('?');


end

transmission
collision
unreachable
ACK_collision
ACK_unreachable

